
#include <QApplication>
#include <QWidget>
#include <QtDBus>
#include <QDBusConnection>
#include <QAction>
#include <QMenu>
#include <QSystemTrayIcon>
#include <QProgressBar>
#include <QLabel>
#include <QBasicTimer>
#include <QEvent>
#include <QtGui>
#include <QDBusAbstractAdaptor>
#include <QObject>

#include "osd2.h"
// #include <iostream>


Systray::Systray(QWidget *parent) : QWidget(parent)
{
        char str[80];
        const char * kdepath;

        QMenu *trayIconMenu;
        QAction *quitAction;
        trayIconMenu = new QMenu();
        trayIconMenu->setTitle("OSD");

        quitAction = new QAction(tr("&Quit"), this);

        connect(quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
        trayIconMenu->addAction(quitAction);

        QSystemTrayIcon *trayIcon;
        trayIcon = new QSystemTrayIcon;

        kdepath = qgetenv("KDEDIR");
        strcpy (str, kdepath);
        strcat (str, "/share/icons/oxygen/32x32/apps/preferences-desktop-sound.png");
        
//         std::cout << str  << std::endl;
        trayIcon->setIcon(QIcon(str));
        trayIcon->setContextMenu(trayIconMenu);
        trayIcon->show();
}

void Center::recenter(void)
{
    QRect screen = QApplication::desktop()->screen()->rect();
    QRect size = this->geometry();
    move((screen.width()-size.width())/2, (screen.height()-size.height())/15*14);
}

void Center::startTimer(int a)
{
    if (! this->isVisible()) {
        show();
    }
    timer->start(a, this);
}

void Center::timerEvent(QTimerEvent *e)
{
    timer->stop();
    hide();
}

Center::Center(QWidget *parent) : QWidget(parent)
{
        Qt::WindowFlags flags = 0;
        setWindowTitle("center");
        resize(400,60);
        pbar = new QProgressBar(this);
        pbar->setGeometry(66,15,320,25);
        pbar->setValue(50);

        QLabel *label = new QLabel("Volume:", this);
        label->move(12,20);
        
        flags |= Qt::WindowStaysOnTopHint;
        flags |= Qt::FramelessWindowHint;
        flags |= Qt::X11BypassWindowManagerHint;
        setWindowFlags(flags);
        recenter();

        timer = new QBasicTimer();

       this->setVisible(false);
}

TestObject::TestObject(QApplication* application, Center* mw) : QDBusAbstractAdaptor(application), app(application)
{
    int kmixstarted = 0;
    thewindow = mw;
    myapp = application;
    QDBusConnection mdbus = QDBusConnection::sessionBus();
    mdbus.registerObject("/Volcontrol", this, QDBusConnection::ExportAllSlots);
    dbus = new QDBusConnection(mdbus);
    
    /*kmixdbus = new QDBusConnection("org.kde.kmix"); */
    mymixer = new QDBusInterface("org.kde.kmix", "/Mixer0", "org.kde.KMix");
    while (kmixstarted == 0) {
	reply = mymixer->call(QLatin1String("masterDeviceIndex"));
	if (reply.type() == QDBusMessage::ReplyMessage) {
	    QList<QVariant> pot = reply.arguments();
	    masterIdx = pot.at(0).toString();
            kmixstarted = 1;
	}
	else {
	sleep(1);
	}
    }

    reply = mymixer->call(QLatin1String("masterVolume"));
    if (reply.type() == QDBusMessage::ReplyMessage) {
        QList<QVariant> pot = reply.arguments();
        volume = (int) pot.at(0).toULongLong();
    }

}

void TestObject::reshow(short volu)
{
    volume = volu;
    thewindow->pbar->setValue(volu);
    thewindow->startTimer(1800);
}

void TestObject::showme(void)
{
    reply = mymixer->call(QLatin1String("masterVolume"));
    if (reply.type() == QDBusMessage::ReplyMessage) {
        QList<QVariant> pot = reply.arguments();
        volume = (int) pot.at(0).toULongLong();
    }
    thewindow->pbar->setValue(volume);
    thewindow->startTimer(1800);
}

void TestObject::showinc(void)
{
    
    mymixer->call(QLatin1String("increaseVolume"), masterIdx);
    reply = mymixer->call(QLatin1String("masterVolume"));
    if (reply.type() == QDBusMessage::ReplyMessage) {
        QList<QVariant> pot = reply.arguments();
        volume = (int) pot.at(0).toULongLong();
    }
    thewindow->pbar->setValue(volume);
    thewindow->startTimer(1800);
}

void TestObject::showdec(void)
{
    mymixer->call(QLatin1String("decreaseVolume"), masterIdx);
    reply = mymixer->call(QLatin1String("masterVolume"));
    if (reply.type() == QDBusMessage::ReplyMessage) {
        QList<QVariant> pot = reply.arguments();
        volume = (int) pot.at(0).toULongLong();
    }
    thewindow->pbar->setValue(volume);
    thewindow->startTimer(1800);
}


void TestObject::mycheck(void)
{
    //do nothing
}
